#' Read Data from the Shiny App Database
#'
#' This function is used to extract data from the Shiny App Database "data_shm.tar".

#' @param file The path of "data_shm.tar" generated by \code{write_hdf5}.
#' @param prefix A vector of data set identifiers such as \code{c('expr_arab', 'expr_chicken', 'df_pair')}. The vector elements must come from the "data" column in the pairing table that is made when calling \code{write_hdf5}.
#' @return A list of data set and/or the pairing table.

#' @examples

#' # See examples in the function write_hdf5.

#' @author Jianhai Zhang \email{jzhan067@@ucr.edu; zhang.jianhai@@hotmail.com} \cr Dr. Thomas Girke \email{thomas.girke@@ucr.edu}

#' @references
#' SummarizedExperiment: SummarizedExperiment container. R package version 1.10.1 \cr R Core Team (2018). R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. URL https://www.R-project.org/
#' Hervé Pagès (2020). HDF5Array: HDF5 backend for DelayedArray objects. R package version 1.16.1.

#' @export read_hdf5
#' @importFrom SummarizedExperiment rowData assay
#' @importFrom HDF5Array loadHDF5SummarizedExperiment
#' @importFrom utils untar

read_hdf5 <- function(file, prefix) {

  options(stringsAsFactors=FALSE)
  dir <- paste0(tempdir(check=TRUE), '/data_shm')
  if (!dir.exists(dir)) dir.create(dir)
  # Check existence of data in prefix.
  untar(file, c('df_pair_assays.h5', 'df_pair_se.rds'), exdir=dir, tar='tar')
  df.pair <- as.data.frame(assay(loadHDF5SummarizedExperiment(dir=dir, prefix='df_pair_')))
  valid <- prefix[!prefix %in% df.pair$data]
  if (length(valid)==1) if (valid!='df_pair') return("The data defined in 'prefix' is not detected!")
  valid <- valid[valid!='df_pair']
  if (length(valid)>0) return(paste0("The data defined in 'prefix' is not detected: ", paste0(valid, collapse=', '), "!"))
  lis <- list(); for (i in prefix) {
    fil0 <- paste0(i, c('_assays.h5', '_se.rds')) 
    untar(file, fil0, exdir=dir, tar='tar')
    # sys <- system(paste0('tar -xf ', file, ' -C ', dir, ' ', paste0(i, c('_assays.h5', '_se.rds'), collapse=' ')))
    if (sum(file.exists(paste0(dir, '/', fil0)))!=2) { # The data is not detected.
      cat(paste0('This data is not detected: ', i, '!'), '\n')
      dat <- list('none'); names(dat) <- 'none'; lis <- c(lis, dat); next
    }
    dat <- loadHDF5SummarizedExperiment(dir=dir, prefix=paste0(i, '_'))
    SummarizedExperiment::assay(dat) <- as.data.frame(assay(dat))
    assay.na <- names(assay(dat))
    if (!is.null(assay.na)) if (assay.na[1]=='merge') dat <- cbind(assay(dat), rowData(dat))
    if (i=='df_pair') dat <- assay(dat)
    dat <- list(dat); names(dat) <- i; lis <- c(lis, dat)

  }; return(lis)

}





